using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using Infragistics.Samples.Shared.Models;
using Infragistics.Samples.Shared.Extensions;

namespace Infragistics.Samples.Shared.DataProviders
{
    /// <summary>
    /// Represents a data provider for loading info about all countries in the world.
    /// </summary>
    public class WorldDataProvider
    {
        public WorldDataProvider()
        {
            this.WorldData = new List<CountryDataModel>();
            this.LoadingError = null;
        }

        #region Properties

        protected List<CountryDataModel> WorldData;
        protected bool IsLoadedWorldInfoData;
        protected bool IsLoadedWorldFinancialData;
        protected bool IsErrorFound;
        protected Exception LoadingError;
        #endregion
        #region Events
        /// <summary>
        /// Occurs when GetWorldDataAsync() is completed
        /// </summary>
        public event EventHandler<GetWorldDataCompletedEventArgs> GetWorldDataCompleted;

        /// <summary>
        /// Occurs when GetWorldInfoDataAsync() is completed
        /// </summary>
        public event EventHandler<GetWorldDataCompletedEventArgs> GetWorldInfoDataCompleted;

        /// <summary>
        /// Occurs when GetWorldFinancialDataAsync() is completed
        /// </summary>
        public event EventHandler<GetWorldDataCompletedEventArgs> GetWorldFinancialDataCompleted;

        #endregion

        #region Methods
        private bool IsWorldDataLoaded()
        {
            if (WorldDataManager.DataSource.Any())
            {
                this.WorldData = this.WorldData = WorldDataManager.DataSource;
                return true;
            }
            return false;
        }
        /// <summary>
        /// Gets World Country (Info and Financial) data from the following files using the XmlDataProvider:
        /// <para>- CountryData.xml </para> 
        /// <para>- GdpByCountry.xml </para> 
        /// </summary>
        public void GetWorldDataAsync()
        {
            if (IsWorldDataLoaded())
            {
                this.ValidateEvents(); return;
            }
            this.IsErrorFound = false;
            this.IsLoadedWorldFinancialData = false;
            this.IsLoadedWorldInfoData = false;

            this.GetWorldFinancialDataAsync();
            this.GetWorldInfoDataAsync();
        }
        /// <summary>
        /// Gets World Financial data from GdpByCountry.xml file using the XmlDataProvider
        /// </summary>
        public void GetWorldFinancialDataAsync()
        {
            if (IsWorldDataLoaded())
            {
                this.ValidateEvents(); return;
            }
         
            this.IsErrorFound = false;
            this.IsLoadedWorldFinancialData = false;

            var worldFinancialDataProvider = new XmlDataProvider();
            worldFinancialDataProvider.GetXmlDataCompleted += OnWorldFinancialXmlDataCompleted;
            worldFinancialDataProvider.GetXmlDataAsync("GdpByCountry.xml");
        }

        /// <summary>
        /// Get World Info data from CountryData.xml file using the XmlDataProvider
        /// </summary>
        public void GetWorldInfoDataAsync()
        {
            if (IsWorldDataLoaded())
            {
                this.ValidateEvents(); return;
            }
         
            this.IsErrorFound = false;
            this.IsLoadedWorldInfoData = false;

            var worldInfoDataProvider = new XmlDataProvider();
            worldInfoDataProvider.GetXmlDataCompleted += OnGetWorldInfoXmlDataCompleted;
            worldInfoDataProvider.GetXmlDataAsync("CountryData.xml");
        }

        private void ValidateEvents()
        {
            if (this.LoadingError != null)
            {
                // notify about the loading error
                OnLoadingError(this.LoadingError);
            }
            if (IsLoadedWorldInfoData && this.GetWorldInfoDataCompleted != null)
            {
                // notify about successful loading of WorldInfoData
                this.GetWorldInfoDataCompleted(this, new GetWorldDataCompletedEventArgs(this.WorldData));
            }
            if (IsLoadedWorldFinancialData && this.GetWorldFinancialDataCompleted != null)
            {
                this.GetWorldFinancialDataCompleted(this, new GetWorldDataCompletedEventArgs(this.WorldData));
            }
            if (IsLoadedWorldInfoData && IsLoadedWorldFinancialData)
            {
                if (this.GetWorldDataCompleted != null)
                {
                    this.GetWorldDataCompleted(this, new GetWorldDataCompletedEventArgs(this.WorldData));
                }
            }
        }
        private void OnLoadingError(Exception error)
        {
            if (this.GetWorldDataCompleted != null)
            {
                this.GetWorldDataCompleted(this, new GetWorldDataCompletedEventArgs(error));
            }
            if (this.GetWorldInfoDataCompleted != null)
            {
                this.GetWorldInfoDataCompleted(this, new GetWorldDataCompletedEventArgs(error));
            }
            if (this.GetWorldFinancialDataCompleted != null)
            {
                this.GetWorldFinancialDataCompleted(this, new GetWorldDataCompletedEventArgs(error));
            }
        }
        private XDocument ReadXmlData(string xmlString)
        {
            XDocument dataSource = new XDocument();
            using (StringReader reader = new StringReader(xmlString))
            {
                string resultString = reader.ReadToEnd();
                if (!string.IsNullOrEmpty(resultString))
                {
                    dataSource = XDocument.Parse(resultString);
                }
            }
            return dataSource;
        }
        /// <summary>
        /// Updates CountryDataModel in the WorldData list with data from passed CountryDataModel
        /// based on CountryCode of CountryDataModel
        /// </summary>
        /// <param name="newCountry"></param>
        private void UpdateWorldData(CountryDataModel newCountry)
        {
            foreach (CountryDataModel country in WorldData)
            {
                if (country.CountryCode.Equals(newCountry.CountryCode))
                {
                    country.UpdateWith(newCountry);
                    break;
                }
            }

        }

        public List<CountryDataModel> MergeWorldData(List<CountryDataModel> data1, List<CountryDataModel> data2)
        {
            foreach (CountryDataModel newCountry in data2)
            {
                foreach (CountryDataModel country in data1)
                {
                    if (country.CountryCode.Equals(newCountry.CountryCode))
                    {
                        country.UpdateWith(newCountry);
                        break;
                    }
                }
            }
            return data1;
        }


        #endregion

        #region Parser Methods
        public List<CountryDataModel> ParseCountryData(string xmlString)
        {
            XDocument xmlDocument = ReadXmlData(xmlString);
            return ParseCountryData(xmlDocument);
        }
        public List<CountryDataModel> ParseCountryData(XDocument xmlDocument)
        {
            IEnumerable<CountryDataModel> results =
           (from d in xmlDocument.Element("NewDataSet").Elements("Table")
            select new CountryDataModel
            {   // get values using XElementExtension
                CountryCode = d.Element("CountryCode").GetString(),
                CountryName = d.Element("CountryName").GetString(),
                BirthRate = d.Element("BirthRate").GetDouble(),
                ElectricityProduction = d.Element("ElectricityProduction").GetDouble(),
                InternetUsers = d.Element("InternetUsers").GetDouble(),
                MedianAge = d.Element("MedianAge").GetDouble(),
                OilProduction = d.Element("OilProduction").GetDouble(),
                Population = d.Element("Population").GetDouble(),
                PublicDebt = d.Element("PublicDebt").GetDouble(),
                Televisions = d.Element("Televisions").GetDouble(),
                UnemploymentRate = d.Element("UnemploymentRate").GetDouble()
            });

            List<CountryDataModel> newWorldData = results.Select(dp => new CountryDataModel
            {
                CountryCode = dp.CountryCode,
                CountryName = dp.CountryName,
                BirthRate = dp.BirthRate,
                ElectricityProduction = dp.ElectricityProduction,
                InternetUsers = dp.InternetUsers,
                MedianAge = dp.MedianAge,
                OilProduction = dp.OilProduction,
                Population = dp.Population,
                PublicDebt = dp.PublicDebt,
                Televisions = dp.Televisions,
                UnemploymentRate = dp.UnemploymentRate
            }).ToList();

            return newWorldData;
        }

        public List<CountryDataModel> ParseGdpByCountryData(string xmlString)
        {
            XDocument xmlDocument = ReadXmlData(xmlString);
            return ParseGdpByCountryData(xmlDocument);
        }
        public List<CountryDataModel> ParseGdpByCountryData(XDocument xmlDocument)
        {

            IEnumerable<CountryDataModel> results =
           (from d in xmlDocument.Element("NewDataSet").Elements("GDPByCountry")
            select new CountryDataModel
            {   // get values using XElementExtension
                CountryCode = d.Element("CountryCode").GetString(),
                GdpPerCapita = d.Element("GDPPerCapita").GetDouble()
            });

            List<CountryDataModel> newWorldData = results.Select(dp => new CountryDataModel
            {
                CountryCode = dp.CountryCode,
                GdpPerCapita = dp.GdpPerCapita,
            }).ToList();

            return newWorldData;
        }

        #endregion

        #region Event Handlers

        /// <summary>
        /// Handles the GetXmlDataCompleted event and parses the country data.
        /// </summary>
        private void OnGetWorldInfoXmlDataCompleted(object sender, GetXmlDataCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                this.LoadingError = e.Error;
            }
            else
            {
                List<CountryDataModel> newWorldData = ParseCountryData(e.Result);
                // add new world data or merged it with existing data
                this.WorldData = this.WorldData.Count == 0 ? newWorldData : this.MergeWorldData(this.WorldData, newWorldData);
                this.IsLoadedWorldInfoData = true;
            }

            this.ValidateEvents();
        }
        /// <summary>
        /// Handles the GetXmlDataCompleted event and parses the country data.
        /// </summary>
        private void OnWorldFinancialXmlDataCompleted(object sender, GetXmlDataCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                this.LoadingError = e.Error;
            }
            else
            {
                List<CountryDataModel> newWorldData = ParseGdpByCountryData(e.Result);
                // add new world data or merged it with existing data
                this.WorldData = this.WorldData.Count == 0 ? newWorldData : this.MergeWorldData(this.WorldData, newWorldData);
                this.IsLoadedWorldFinancialData = true;
            }
            this.ValidateEvents();
        }


        #endregion


    }
    public class GetWorldDataCompletedEventArgs : EventArgs
    {

        public GetWorldDataCompletedEventArgs(List<CountryDataModel> data)
        {
            this.Result = data;
            this.Error = null;
        }

        public GetWorldDataCompletedEventArgs(Exception error)
        {
            this.Result = null;
            this.Error = error;
        }
        public List<CountryDataModel> Result { get; set; }
        public Exception Error { get; private set; }

    }
    public static class WorldDataManager
    {
        static WorldDataManager()
        {
            DataSource = new List<CountryDataModel>();
        }
        public static List<CountryDataModel> DataSource;
    
    }
}