using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;

 
namespace Infragistics.Samples.Shared.Models
{
    public class StockMarketDataSample : StockMarketDataCollection
    {
        
    }
    public class StockMarketDataCollection : ObservableCollection<StockMarketDataPoint>
    {
        public StockMarketDataCollection()
        {
            _stockSettings = new StockMarketSettings();
            this.Generate();
        }
        public void Generate()
        {
            this.Clear();
            StockMarketGenerator.Settings = this.Settings;
            StockMarketGenerator.Generate();
            foreach (var dp in StockMarketGenerator.Data)
            {
                this.Add(dp);
            }

            LastItem = this[this.Count - 1];
        }

        private StockMarketDataPoint _lastItem;
        public StockMarketDataPoint LastItem
        {
            get { return _lastItem; }
            set { _lastItem = value; OnPropertyChanged(new PropertyChangedEventArgs("LastItem")); }
        }

        private StockMarketSettings _stockSettings;
       

        /// <summary>
        /// Settings used to generate StockMarketDataPoint objects
        /// </summary>
        public StockMarketSettings Settings
        {
            get { return _stockSettings; }
            set
            {
                _stockSettings = value;
                this.Generate();
                OnPropertyChanged(new PropertyChangedEventArgs("Settings"));
            }
        }

    }
    public class StockMarketSettings : DataSettings
    {
        public StockMarketSettings()
        {
            DataPoints = 100;

            VolumeStart = 2000;
            VolumeRange = 500;
            VolumeSample = 4;

            PriceStart = 1000;
            PriceRange = 50;
            PriceSample = 4;
             
            DateStart = new DateTime(2010, 1, 1);
            DateInterval = TimeSpan.FromDays(1);
        }

        #region Properties
        /// <summary>
        /// Determines range for used to generate random Volume value 
        /// </summary>
        public double VolumeRange { get; set; }
        /// <summary>
        /// Determines number of samples used to generate random Volume values
        /// </summary>
        public int VolumeSample { get; set; }
        /// <summary>
        /// Determines starting Volume value for the first StockMarketDataPoint object
        /// </summary>
        public double VolumeStart { get; set; }

        /// <summary>
        /// Determines range used to generate random Open, High, Low, and Close values
        /// </summary>
        public double PriceRange { get; set; }
        /// <summary>
        /// Determines number of samples used value used to generate random Price values
        /// </summary>
        public int PriceSample { get; set; }
        /// <summary>
        /// Determines starting Price value for the first StockMarketDataPoint object
        /// </summary>
        public double PriceStart { get; set; }

        /// <summary>
        /// Determines time intervals between StockMarketDataPoint objects
        /// </summary>
        public TimeSpan DateInterval { get; set; }
        /// <summary>
        /// Determines starting date value for the first StockMarketDataPoint object
        /// </summary>
        public DateTime DateStart { get; set; }
        #endregion
    }
    public class StockMarketDataPoint : ObservableModel
    {
        #region Constructors
        public StockMarketDataPoint()
            : this(new DateTime(2010, 1, 1), 100, 120, 90, 110, 1000)
        { }

        public StockMarketDataPoint(DateTime date, double open, double high, double low, double close, double volume)
        {
            _date = date;
            _open = open;
            _high = high;
            _low = low;
            _close = close;
            _volume = volume;
        }
        public StockMarketDataPoint(string date, double open, double high, double low, double close, double volume)
            : this(DateTime.Parse(date, System.Globalization.CultureInfo.InvariantCulture),
                open, high, low, close, volume)
        { }

        public StockMarketDataPoint(StockMarketDataPoint dataPoint)
        {
            _date = dataPoint.Date;
            _open = dataPoint.Open;
            _high = dataPoint.High;
            _low = dataPoint.Low;
            _close = dataPoint.Close;
            _volume = dataPoint.Volume;
            _index = dataPoint.Index;
        }
        #endregion

        #region Properties
        private double _open;
        private double _high;
        private double _low;
        private double _close;
        private double _volume;
        private DateTime _date;
        private int _index;
        private string _label;

        public int Index { get { return _index; } set { if (_index == value) return; _index = value; OnPropertyChanged("Index"); } }
        public double Open { get { return _open; } set { if (_open == value) return; _open = value; OnPropertyChanged("Open"); } }
        public double High { get { return _high; } set { if (_high == value) return; _high = value; OnPropertyChanged("High"); } }
        public double Low { get { return _low; } set { if (_low == value) return; _low = value; OnPropertyChanged("Low"); } }
        public double Close { get { return _close; } set { if (_close == value) return; _close = value; OnPropertyChanged("Close"); } }
        public double Volume { get { return _volume; } set { if (_volume == value) return; _volume = value; OnPropertyChanged("Volume"); } }
        public DateTime Date { get { return _date; } set { if (_date == value) return; _date = value; OnPropertyChanged("Date"); } }

        public string Category { get { return _label; } set { if (_label == value) return; _label = value; OnPropertyChanged("Category"); } }




        /// <summary>
        /// returns the difference between Close and Open values
        /// </summary>
        public double Change
        {
            get { return Close - Open; }
        }
        public double ChangePerCent
        {
            get { return (Change / Open) * 100; }
        }
        #endregion

        public new string ToString()
        {
            return String.Format(
                "Index {0}, Open {1}, High {2}, Low {3}, Close {4}, Change {5}, Volume {6}, Date {7}",
                Index, Open, High, Low, Close, Change, Volume, Date);
        }

    }
    public static class StockMarketGenerator
    {
        static StockMarketGenerator()
        {
            Generator = new Random();
            Settings = new StockMarketSettings();
            Data = GenerateData();
        }
        #region Properties
        public static StockMarketSettings Settings { get; set; }
        public static List<StockMarketDataPoint> Data { get; set; }
        public static StockMarketDataPoint LastDataPoint { get; set; }

        #endregion
        #region Fields
        private static readonly Random Generator;
        #endregion
        #region Methods
        public static void Generate()
        {
            Data = GenerateData();
        }
        public static DateTime GenerateStockDate(DateTime lastDate)
        {
            return lastDate.Add(Settings.DateInterval);
        }

        #region Methods for Random GeneratorMode


        public static double GenerateStockVolume(double preVolume)
        {
            var sum = 0.0;
            //var min = (int)preVolume - Settings.VolumeRange;
            //var max = (int)preVolume + Settings.VolumeRange;
            var min = Settings.VolumeStart - Settings.VolumeRange;
            var max = Settings.VolumeStart + Settings.VolumeRange;

            for (var i = 0; i < Settings.VolumeSample; i++)
            {
                sum += Generator.Next((int)min, (int)max);
            }
            var volume = sum / Settings.VolumeSample;
            return Math.Abs(volume);
        }

        public static double CurrentRandomValue;
        public static double GenerateStockRandom()
        {
            double nextRandomValue = Generator.NextDouble();
            if (nextRandomValue > .5)
                CurrentRandomValue += nextRandomValue * Settings.PriceRange;
            else
                CurrentRandomValue -= nextRandomValue * Settings.PriceRange;
            return CurrentRandomValue;
        }
        public static double GenerateStockOpen(double preClose)
        {
            //open value always equals to previous close value
            return preClose;
        }
        public static double GenerateStockOpen(double low, double high)
        {
            low = Math.Max(low, Settings.PriceStart - Settings.PriceRange);
            high = Math.Min(high, Settings.PriceStart + Settings.PriceRange);
            
            var min = (int)Math.Floor(Math.Min(low, high));
            var max = (int)Math.Ceiling(Math.Max(low, high));
            return Math.Abs(Generator.Next(min, max));
            // or return (low + high) / 2.0;
        }
        public static double GenerateStockHigh(double open)
        {
            var sum = 0.0;
            var min = (int)open;
            var max = (int)open + Settings.PriceRange;
            for (int i = 0; i < Settings.PriceSample; i++)
            {
                sum += Generator.Next((int)min, (int)max);
            }
            return Math.Abs(sum / Settings.PriceSample);
        }
        public static double GenerateStockLow(double open)
        {
            var sum = 0.0;
            var min = (int)open - Settings.PriceRange;
            var max = (int)open;
            for (var i = 0; i < Settings.PriceSample; i++)
            {
                sum += Generator.Next((int)min, (int)max);
            }
            return Math.Abs(sum / Settings.PriceSample);
        }
        public static double GenerateStockClose(double low, double high)
        {
            var min = (int)Math.Floor(Math.Min(low, high));
            var max = (int)Math.Ceiling(Math.Max(low, high));
            return Math.Abs((double)Generator.Next(min, max));
            // or return (low + high) / 2.0;
        }


        #endregion

        /// <summary>
        /// Generate new StockMarketDataPoint based on the passed StockMarketDataPoint
        /// </summary>
        /// <param name="dataPoint"></param>
        /// <returns></returns>
        public static StockMarketDataPoint GenerateDataPoint(StockMarketDataPoint dataPoint)
        {
            var open = GenerateStockOpen(dataPoint.Close);
            //double open = GenerateStockOpen(dataPoint.Low, dataPoint.High);
            //double open = GenerateStockRandom();
            var high = GenerateStockHigh(open);
            var low = GenerateStockLow(open);
            var close = GenerateStockClose(low, high);

            var volume = GenerateStockVolume(dataPoint.Volume);
            var date = dataPoint.Date.Add(Settings.DateInterval);
            var index = dataPoint.Index + 1;

            LastDataPoint = new StockMarketDataPoint
            {
                Index = index,
                Date = date,
                Open = open,
                Close = close,
                High = high,
                Low = low,
                Volume = volume
            };
            return LastDataPoint;
        }

        /// <summary>
        /// Generate new StockMarketDataPoint based on the last StockMarketDataPoint in List of StockMarketDataPoint 
        /// </summary>
        /// <returns></returns>
        public static StockMarketDataPoint GenerateDataPoint()
        {
            var newDataPoint = GenerateDataPoint(LastDataPoint);
            return newDataPoint;
        }

        /// <summary>
        /// Generate List of StockMarketDataPoint based on the Settings
        /// </summary>
        /// <returns></returns>
        public static List<StockMarketDataPoint> GenerateData()
        {
            var data = new List<StockMarketDataPoint>();

            var dataPoint = new StockMarketDataPoint
            {
                Index = -1,
                Close = Settings.PriceStart,
                Volume = Settings.VolumeStart,
                Date = Settings.DateStart
            };
            CurrentRandomValue = Settings.PriceStart;

            for (int i = 0; i < Settings.DataPoints; i++)
            {
                dataPoint = GenerateDataPoint(dataPoint);
                data.Add(dataPoint);
            }
            return data;
        }

        /// <summary>
        /// Append new StockMarketDataPoint to existing StockMarketData
        /// </summary>
        public static void AppendDataPoint()
        {
            Data.Add(GenerateDataPoint());
        }

        #endregion

    }
}