using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Xml.Linq;
using Infragistics.Samples.Shared.DataProviders;
using Infragistics.Samples.Shared.Extensions;

namespace Infragistics.Samples.Shared.Models
{
    /// <summary>
    /// Represents an EarthQuake data model in geographic context: <seealso cref="GeoLocation"/>
    /// </summary>
    public class EarthQuake : GeoLocation
    {
        public EarthQuake()
        {
            this.Magnitude = 0.0;
            this.Depth = 0.0;
        }
        public double Depth { get; set; }
        public double Magnitude { get; set; }
    }

    /// <summary>
    /// Represents a a collection of <seealso cref="EarthQuake"/> data models.
    /// </summary>
    public class EarthquakeCollection : ObservableCollection<EarthQuake>
    {
        /// <summary>
        /// Occurs when data loading has completed
        /// </summary>
        public event EventHandler<EventArgs> LoadingDataCompleted;
        protected void OnLoadingDataCompleted(EventArgs eventArgs)
        {
            if (this.LoadingDataCompleted != null)
            {
                this.LoadingDataCompleted(this, eventArgs);
            }
        }
        public void LoadHistoricData()
        {
            Load("EarthQuakes.xml");
        }
       
        /// <summary>
        /// Loads GeoPlace data from a given Xml file  
        /// <para> Format: event lat="18.871" lng="146.2927" dkm="99.9" mag="5.7" source="NEIC" date="2011/06/26 09:19:48.2000"</para>
        /// </summary>
        /// <param name="xmlFileName"></param>
        public void Load(string xmlFileName)
        {
            // create data provider for reading data from Xml file
            this.XmlDataProvider = new XmlDataProvider();
            this.XmlDataProvider.GetXmlDataCompleted += OnGetXmlDataCompleted;
            this.XmlDataProvider.GetXmlDataAsync(xmlFileName);
        }
        protected XmlDataProvider XmlDataProvider;
        private void OnGetXmlDataCompleted(object sender, GetXmlDataCompletedEventArgs e)
        {
            if (e.Error != null) return;

            this.Clear();

            var data = ParseEarthQuakes(e.Result);
            foreach (var earthQuake in data)
            {
                this.Add(earthQuake);
            }

            OnLoadingDataCompleted(EventArgs.Empty);
        }
        public static List<EarthQuake> ParseEarthQuakes(XDocument xmlDocument)
        {
            List<EarthQuake> results =
               (from d in xmlDocument.Element("events").Elements("event")
                select new EarthQuake
                {
                    // get values using XElementExtension            
                    Name = d.Attribute("date").GetString(),
                    Longitude = d.Attribute("lng").GetDouble(),
                    Latitude = d.Attribute("lat").GetDouble(),
                }).ToList();

            return results;
        }

  

    }
}